@@ -20,6 +20,7 @@ from .chars import (
     TIMES,
     WHSP,
 )
+from .constants import RU_LONG_PREPOSICIONES, RU_PARTICLES
 from .utils import map_choices
 
 __all__ = ('EnQuotes', 'RuQuotes', 'EnRuExpressions', 'RuExpressions')
@@ -437,26 +438,26 @@ class EnRuExpressions:
 
 
 class RuExpressions(EnRuExpressions):
-    expressions = EnRuExpressions.expressions + ('rep_positional_spaces_for_words',)
+    expressions = EnRuExpressions.expressions + (
+        'rep_positional_spaces_after_long_preposiciones',
+        'rep_positional_spaces_before_particles',
+    )
 
-    rep_positional_spaces_for_words = {
-        'after': (
-            'без',
-            'перед',
-            'при',
-            'через',
-            'над',
-            'под',
-            'про',
-            'для',
-            'около',
-            'среди',
-        )
-    }
-
-    def expr_rep_positional_spaces_for_words(self):
+    def expr_rep_positional_spaces_after_long_preposiciones(self):
         """
         Replaces whitespaces after and before certain words with non-breakable space.
         """
-        after_words = self.rep_positional_spaces_for_words['after']
-        return [(fr'(?<={word}){WHSP}+', NBSP) for word in after_words]
+        return [(fr'(\W{word}){WHSP}+', fr'\1{NBSP}') for word in RU_LONG_PREPOSICIONES]
+
+    def expr_rep_positional_spaces_before_particles(self):
+        """Replaces whitespaces before particles with non-breakable space."""
+        before_particles = [
+            (fr'{WHSP}+({word}\W)', fr'{NBSP}\1') for word in RU_PARTICLES
+        ]
+        # Remove effect of `expr_pairs` for short particles
+        short_particles = [word for word in RU_PARTICLES if len(word) < 3]
+        after_short_particles = [
+            (fr'(\W{word}){NBSP}+', fr'\1{WHSP}') for word in short_particles
+        ]
+
+        return [*before_particles, *after_short_particles]
